import os
import logging
from django.core.management.base import BaseCommand

from mglib.pdfinfo import get_pagecount

logger = logging.getLogger(__name__)


class Command(BaseCommand):

    help = """
    Calls mglib.pdfinfo.get_pagecount(path) on given file path
    Useful for debugging purposes.
"""

    def add_arguments(self, parser):
        parser.add_argument(
            'file_path',
            help="Path to a file on local file system"
        )

    def handle(self, *args, **options):
        file_path = options.get(
            'file_path',
            False
        )

        if not os.path.exists(file_path):
            logger.debug(f"Path {file_path} does not exit. Quit.")
            return

        page_count = get_pagecount(file_path)

        logger.debug(f"Page count={page_count}")



import os
import logging


logger = logging.getLogger(__name__)

SAFE_EXTENSIONS = [
    '.svg',
    '.txt',
    '.jpg',
    '.jpeg',
    '.png',
    '.hocr',
    '.pdf',
    '.tiff'
]


def get_bool(key, default="NO"):
    """
    Returns True if environment variable named KEY is one of
    "yes", "y", "t", "true" (lowercase of uppercase)

    otherwise returns False
    """
    env_var_value = os.getenv(key, default).lower()
    YES_VALUES = ("yes", "y", "1", "t", "true")
    if env_var_value in YES_VALUES:
        return True

    return False


def safe_to_delete(place):
    if not os.path.exists(place):
        logging.warning(
            f"Trying to delete not exising folder"
            f" {place}"
        )
        return False

    for root, dirs, files in os.walk(place):
        for name in files:
            base, ext = os.path.splitext(name)
            if ext.lower() not in SAFE_EXTENSIONS:
                logger.warning(
                    f"Trying to delete unsefe location: "
                    f"extention={ext} not found in {SAFE_EXTENSIONS}"
                )
                return False

    return True


def get_reordered_list(pages_data, page_count):
    """
    Returns a list of integers. Each number in the list
    is correctly positioned (newly ordered) page.
    Examples:
    If in document with 4 pages first and second pages were
    swapped, then returned list will be:
        [2, 1, 3, 4]
    If first page was swapped with last one (also 4 paegs document)
    result list will look like:
        [4, 2, 3, 1]
    """
    results = []
    page_map = {number: number for number in range(1, page_count + 1)}

    for item in pages_data:
        k = int(item['old_number'])
        v = int(item['new_number'])
        page_map[k] = v

    for number in range(1, page_count + 1):
        results.append(
            page_map[number]
        )

    return results


def annotate_page_data(pages, pages_data, field='angle'):
    """
    Returns a list of dictionaries containing objects with two keys:
        - number
        - ``field``

    ``number`` is extracted from ``pages`` queryset.
    ``filed`` is extracted from pages_data.

    :param pages: Pages queryset
    :param pages_data: list of dictionaries. Each dictionary contains
    key 'id' and ``field``.
    """
    ret = []
    for page in pages:
        page_dict = {}
        page_dict['number'] = page.number
        for page_data in pages_data:
            if str(page.id) == str(page_data['id']):
                page_dict[field] = page_data[field]

        ret.append(page_dict)

    return ret



import os
import subprocess

from django.core.checks import Warning, register

from .app_settings import settings


USED_BINARIES = {
    settings.BINARY_FILE: {
        "msg": (
            "Without it, Papermerge won't be"
            " "
            "able to learn file mime type"),
        "option": "-v"
    },
    settings.BINARY_CONVERT: {
        "msg": (
            "Without it, image resizing is not possible"
        ),
        "option": "-v"
    },
    settings.BINARY_OCR: {
        "msg": (
            "Without it, OCR of the documents is impossible"
        ),
        "option": "-v"
    },
    settings.BINARY_IDENTIFY: {
        "msg": (
            "Without it, it is not possible to count pages in TIFF"
        ),
        "option": "-v"
    },
}


@register()
def binaries_check(app_configs, **kwargs):
    """
    Papermerge requires the existence of a few binaries, so it checks
    if required binaries are available.

    See settings prefixed with BINARY_ defined in mglib.conf.default_settings
    for full list of dependencies.
    """
    error = "Papermerge can't find {}. {}."
    hint = "Either it's not in your PATH or it's not installed."

    check_messages = []
    for binary_path in USED_BINARIES.keys():

        binary = os.path.basename(binary_path)
        option = USED_BINARIES[binary_path]["option"]
        try:
            subprocess.run(
                [binary_path, option],
                stdout=subprocess.DEVNULL,
                stderr=subprocess.DEVNULL
            )
        except FileNotFoundError:
            check_messages.append(
                Warning(
                    error.format(binary, USED_BINARIES[binary_path]["msg"]),
                    hint
                )
            )

    return check_messages


"""
@register()
def imap_login_check(app_configs, **kwargs):

    host = settings.IMPORT_MAIL_HOST
    user = settings.IMPORT_MAIL_USER
    password = settings.IMPORT_MAIL_PASS

    check_messages = []
    msg = f"Failed to login to IMAP server '{host}'"
    hint =
        Please double check that IMPORT_MAIL_HOST,
        IMPORT_MAIL_USER, IMPORT_MAIL_PASS settings
        are correct.

    if all([host, user, password]):
        try:
            server = imap_login(
                imap_server=host,
                username=user,
                password=password
            )
        except Exception:
            server = None

        if not server:
            check_messages.append(
                Warning(
                    msg,
                    hint
                )
            )

                Warning(
                    msg,
                    hint
                )
            )

    return check_messages
"""



"""
Global Papermerge exception and warning classes.
"""
from rest_framework.exceptions import APIException


class APIBadRequest(APIException):
    status_code = 400


class SuperuserDoesNotExist(Exception):
    """
    Raised when superuser was not found.
    Papermerge must have at least one superuser.
    """
    pass


class FileTypeNotSupported(Exception):
    """File type not supported"""
    pass



from rest_framework.serializers import Serializer
from rest_framework_json_api import serializers
from drf_spectacular.utils import OpenApiExample

from papermerge.core.models import User
from .permission import PermissionSerializer
from .group import GroupSerializer


class UserSerializer(serializers.ModelSerializer):
    user_permissions = PermissionSerializer(many=True, read_only=True)
    perm_codenames = serializers.ListField(
        child=serializers.CharField(max_length=200),
        read_only=True
    )

    included_serializers = {
        'groups': GroupSerializer
    }

    class Meta:
        model = User
        resource_name = 'users'
        fields = (
            'username',
            'inbox_folder',
            'home_folder',
            'first_name',
            'last_name',
            'email',
            'is_active',
            'is_staff',
            'is_superuser',
            'date_joined',
            'user_permissions',
            'groups',
            'perm_codenames'
        )
        read_only_fields = ('inbox_folder', 'home_folder', 'date_joined')


data_user_response_example = OpenApiExample(
    'Response example',
    summary='Current user response example',
    description='Please add description',
    value={
        'data': {
            'type': 'users',
            'id': '0d76a122-629d-401a-b122-55c7afabd570',
            'attributes': {
                'username': 'eugen',
                'first_name': '',
                'last_name': '',
                'email': 'eugen@mail.com',
                'is_active': True,
                'is_staff': True,
                'is_superuser': True,
                'date_joined': '2022-06-08T06:15:05.855523+02:00',
                'user_permissions': [],
                'perm_codenames': []
            },
        },
        'relationships': {
            'inbox_folder': {
                'data': {
                    'type': "folders",
                    'id': '70a2e21f-2786-422c-9353-ede77cc8aab2'
                }
            },
            'home_folder': {
                'data': {
                    'type': 'folders',
                    'id': 'b4aaabe3-a8f1-48bf-8d10-b191a0d4ffbd'
                }
             },
            'groups': {
                'data': [],
                'meta': {
                    'count': 0
                },
            }
        }
    }
)


class Data_UserSerializer(Serializer):
    data = UserSerializer()
